@@ -16,13 +16,13 @@
 %                                April 2004                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -102,7 +102,7 @@ ModuleExport size_t RegisterCIPImage(void)
   entry->encoder=(EncodeImageHandler *) WriteCIPImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Cisco IP phone image format");
-  entry->module=ConstantString("CIP");
+  entry->magick_module=ConstantString("CIP");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
