@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -214,14 +214,14 @@ ModuleExport size_t RegisterINLINEImage(void)
   entry->encoder=(EncodeImageHandler *) WriteINLINEImage;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Base64-encoded inline images");
-  entry->module=ConstantString("INLINE");
+  entry->magick_module=ConstantString("INLINE");
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("INLINE");
   entry->decoder=(DecodeImageHandler *) ReadINLINEImage;
   entry->encoder=(EncodeImageHandler *) WriteINLINEImage;
   entry->format_type=ImplicitFormatType;
   entry->description=ConstantString("Base64-encoded inline images");
-  entry->module=ConstantString("INLINE");
+  entry->magick_module=ConstantString("INLINE");
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
