@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -142,8 +142,7 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static Image *ReadTGAImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadTGAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
   Image
     *image;
@@ -243,9 +242,13 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
   */
   image->columns=tga_info.width;
   image->rows=tga_info.height;
-  alpha_bits=(tga_info.attributes & 0x0FU);
-  image->matte=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
-    (tga_info.colormap_size == 32) ?  MagickTrue : MagickFalse;
+  if ((tga_info.image_type != TGAMonochrome) &&
+      (tga_info.image_type != TGARLEMonochrome))
+    {
+      alpha_bits=(tga_info.attributes & 0x0FU);
+      image->matte=(alpha_bits > 0) || (tga_info.bits_per_pixel == 32) ||
+        (tga_info.colormap_size == 32) ?  MagickTrue : MagickFalse;
+    }
   if ((tga_info.image_type != TGAColormap) &&
       (tga_info.image_type != TGARLEColormap))
     image->depth=(size_t) ((tga_info.bits_per_pixel <= 8) ? 8 :
@@ -254,10 +257,14 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
     image->depth=(size_t) ((tga_info.colormap_size <= 8) ? 8 :
       (tga_info.colormap_size <= 16) ? 5 : 8);
   if ((tga_info.image_type == TGAColormap) ||
-      (tga_info.image_type == TGAMonochrome) ||
-      (tga_info.image_type == TGARLEColormap) ||
-      (tga_info.image_type == TGARLEMonochrome))
+      (tga_info.image_type == TGARLEColormap))
     image->storage_class=PseudoClass;
+  if ((tga_info.image_type == TGAMonochrome) ||
+      (tga_info.image_type == TGARLEMonochrome))
+    {
+      image->type=GrayscaleType;
+      image->colorspace=GRAYColorspace;
+    }
   image->compression=NoCompression;
   if ((tga_info.image_type == TGARLEColormap) ||
       (tga_info.image_type == TGARLEMonochrome) ||
@@ -274,7 +281,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
 
           one=1;
           image->colors=one << tga_info.bits_per_pixel;
-          if (image->colors > GetBlobSize(image))
+          if ((MagickSizeType) image->colors > GetBlobSize(image))
             ThrowReaderException(CorruptImageError,
               "InsufficientImageDataInFile");
           if (AcquireImageColormap(image,image->colors) == MagickFalse)
@@ -310,16 +317,16 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
   if (tga_info.attributes & (1UL << 4))
     {
       if (tga_info.attributes & (1UL << 5))
-        SetImageArtifact(image,"tga:image-origin","TopRight");
+        image->orientation=TopRightOrientation;
       else
-        SetImageArtifact(image,"tga:image-origin","BottomRight");
+        image->orientation=BottomRightOrientation;
     }
   else
     {
       if (tga_info.attributes & (1UL << 5))
-        SetImageArtifact(image,"tga:image-origin","TopLeft");
+        image->orientation=TopLeftOrientation;
       else
-        SetImageArtifact(image,"tga:image-origin","BottomLeft");
+        image->orientation=BottomLeftOrientation;
     }
   if (image_info->ping != MagickFalse)
     {
@@ -459,7 +466,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
             index=(Quantum) pixels[0];
             if (tga_info.colormap_type != 0)
               pixel=image->colormap[(ssize_t) ConstrainColormapIndex(image,
-                1UL*index)];
+                (ssize_t) index)];
             else
               {
                 pixel.red=ScaleCharToQuantum((unsigned char) index);
@@ -490,7 +497,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
               pixel.opacity=(k & 0x80) == 0 ? (Quantum) TransparentOpacity :
                 (Quantum) OpaqueOpacity;
             if (image->storage_class == PseudoClass)
-              index=ConstrainColormapIndex(image,((size_t) k << 8)+j);
+              index=ConstrainColormapIndex(image,(ssize_t) ((size_t) k << 8)+j);
             break;
           }
           case 24:
@@ -595,7 +602,7 @@ ModuleExport size_t RegisterTGAImage(void)
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->magick_module=ConstantString("TGA");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("TGA");
@@ -603,7 +610,7 @@ ModuleExport size_t RegisterTGAImage(void)
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->magick_module=ConstantString("TGA");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VDA");
@@ -611,7 +618,7 @@ ModuleExport size_t RegisterTGAImage(void)
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->magick_module=ConstantString("TGA");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("VST");
@@ -619,7 +626,7 @@ ModuleExport size_t RegisterTGAImage(void)
   entry->encoder=(EncodeImageHandler *) WriteTGAImage;
   entry->adjoin=MagickFalse;
   entry->description=ConstantString("Truevision Targa image");
-  entry->module=ConstantString("TGA");
+  entry->magick_module=ConstantString("TGA");
   entry->seekable_stream=MagickTrue;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
@@ -836,7 +843,13 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
         else
           tga_info.colormap_size=24;
       }
-  value=GetImageArtifact(image,"tga:image-origin");
+  if ((image->orientation == BottomRightOrientation) ||
+      (image->orientation == TopRightOrientation))
+    tga_info.attributes|=(1UL << 4);
+  if ((image->orientation == TopLeftOrientation) ||
+      (image->orientation == TopRightOrientation))
+    tga_info.attributes|=(1UL << 5);
+  value=GetImageArtifact(image,"tga:image-origin");  /* deprecated */
   if (value != (const char *) NULL)
     {
       OrientationType
@@ -849,6 +862,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
       if (origin == TopLeftOrientation || origin == TopRightOrientation)
         tga_info.attributes|=(1UL << 5);
     }
+  if ((image->columns > 65535) || (image->rows > 65535))
+    ThrowWriterException(ImageError,"WidthOrHeightExceedsLimit");
   /*
     Write TGA header.
   */
