@@ -17,13 +17,13 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    https://www.imagemagick.org/script/license.php                           %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -73,6 +73,42 @@
 #include "lcms2.h"
 #endif
 #endif
+#if defined(MAGICKCORE_XML_DELEGATE)
+#  if defined(MAGICKCORE_WINDOWS_SUPPORT)
+#    if !defined(__MINGW32__)
+#      include <win32config.h>
+#    endif
+#  endif
+#  include <libxml/parser.h>
+#  include <libxml/tree.h>
+#endif
+
+/*
+  Definitions
+*/
+#define LCMSHDRI
+#if !defined(MAGICKCORE_HDRI_SUPPORT)
+  #if (MAGICKCORE_QUANTUM_DEPTH == 8)
+  #undef LCMSHDRI
+  #define LCMSScaleSource(pixel)  ScaleQuantumToShort(pixel)
+  #define LCMSScaleTarget(pixel)  ScaleShortToQuantum(pixel)
+  typedef unsigned short
+    LCMSType;
+  #elif (MAGICKCORE_QUANTUM_DEPTH == 16)
+  #undef LCMSHDRI
+  #define LCMSScaleSource(pixel)  (pixel)
+  #define LCMSScaleTarget(pixel)  (pixel)
+  typedef unsigned short
+    LCMSType;
+  #endif
+#endif
+
+#if defined(LCMSHDRI)
+#define LCMSScaleSource(pixel)  (source_scale*QuantumScale*(pixel))
+#define LCMSScaleTarget(pixel) ClampToQuantum(target_scale*QuantumRange*(pixel))
+typedef double
+  LCMSType;
+#endif
 
 /*
   Forward declarations
@@ -329,23 +365,24 @@ MagickExport char *GetNextImageProfile(const Image *image)
 
 #if defined(MAGICKCORE_LCMS_DELEGATE)
 
-static double **DestroyPixelThreadSet(double **pixels)
+static LCMSType **DestroyPixelThreadSet(LCMSType **pixels)
 {
   register ssize_t
     i;
 
-  assert(pixels != (double **) NULL);
+  if (pixels == (LCMSType **) NULL)
+    return((LCMSType **) NULL);
   for (i=0; i < (ssize_t) GetMagickResourceLimit(ThreadResource); i++)
-    if (pixels[i] != (double *) NULL)
-      pixels[i]=(double *) RelinquishMagickMemory(pixels[i]);
-  pixels=(double **) RelinquishMagickMemory(pixels);
+    if (pixels[i] != (LCMSType *) NULL)
+      pixels[i]=(LCMSType *) RelinquishMagickMemory(pixels[i]);
+  pixels=(LCMSType **) RelinquishMagickMemory(pixels);
   return(pixels);
 }
 
-static double **AcquirePixelThreadSet(const size_t columns,
+static LCMSType **AcquirePixelThreadSet(const size_t columns,
   const size_t channels)
 {
-  double
+  LCMSType
     **pixels;
 
   register ssize_t
@@ -355,15 +392,15 @@ static double **AcquirePixelThreadSet(const size_t columns,
     number_threads;
 
   number_threads=(size_t) GetMagickResourceLimit(ThreadResource);
-  pixels=(double **) AcquireQuantumMemory(number_threads,sizeof(*pixels));
-  if (pixels == (double **) NULL)
-    return((double **) NULL);
+  pixels=(LCMSType **) AcquireQuantumMemory(number_threads,sizeof(*pixels));
+  if (pixels == (LCMSType **) NULL)
+    return((LCMSType **) NULL);
   (void) memset(pixels,0,number_threads*sizeof(*pixels));
   for (i=0; i < (ssize_t) number_threads; i++)
   {
-    pixels[i]=(double *) AcquireQuantumMemory(columns,channels*
+    pixels[i]=(LCMSType *) AcquireQuantumMemory(columns,channels*
       sizeof(**pixels));
-    if (pixels[i] == (double *) NULL)
+    if (pixels[i] == (LCMSType *) NULL)
       return(DestroyPixelThreadSet(pixels));
   }
   return(pixels);
@@ -825,7 +862,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
           GetStringInfoDatum(profile),(cmsUInt32Number)
           GetStringInfoLength(profile));
         if (source_profile == (cmsHPROFILE) NULL)
-          ThrowBinaryException(ResourceLimitError,
+          ThrowBinaryImageException(ResourceLimitError,
             "ColorspaceColorProfileMismatch",name);
         if ((cmsGetDeviceClass(source_profile) != cmsSigLinkClass) &&
             (icc_profile == (StringInfo *) NULL))
@@ -853,18 +890,22 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               source_type,
               target_type;
 
-            double
-              **magick_restrict source_pixels,
-              source_scale,
-              **magick_restrict target_pixels,
-              target_scale;
-
             ExceptionInfo
               *exception;
 
             int
               intent;
 
+            LCMSType
+              **magick_restrict source_pixels,
+              **magick_restrict target_pixels;
+
+#if defined(LCMSHDRI)
+            LCMSType
+              source_scale,
+              target_scale;
+#endif
+
             MagickOffsetType
               progress;
 
@@ -887,103 +928,163 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                   ThrowProfileException(ResourceLimitError,
                     "ColorspaceColorProfileMismatch",name);
               }
+#if defined(LCMSHDRI)
             source_scale=1.0;
+#endif
+            source_colorspace=sRGBColorspace;
             source_channels=3;
             switch (cmsGetColorSpace(source_profile))
             {
               case cmsSigCmykData:
               {
                 source_colorspace=CMYKColorspace;
-                source_type=(cmsUInt32Number) TYPE_CMYK_DBL;
                 source_channels=4;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_CMYK_DBL;
                 source_scale=100.0;
-                break;
-              }
-              case cmsSigLabData:
-              {
-                source_colorspace=LabColorspace;
-                source_type=(cmsUInt32Number) TYPE_Lab_DBL;
-                source_scale=100.0;
+#else
+                source_type=(cmsUInt32Number) TYPE_CMYK_16;
+#endif
                 break;
               }
               case cmsSigGrayData:
               {
                 source_colorspace=GRAYColorspace;
-                source_type=(cmsUInt32Number) TYPE_GRAY_DBL;
                 source_channels=1;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_GRAY_DBL;
+#else
+                source_type=(cmsUInt32Number) TYPE_GRAY_16;
+#endif
                 break;
               }
+              case cmsSigLabData:
+              {
+                source_colorspace=LabColorspace;
+#if defined(LCMSHDRI)
+                source_type=(cmsUInt32Number) TYPE_Lab_DBL;
+                source_scale=100.0;
+#else
+                source_type=(cmsUInt32Number) TYPE_Lab_16;
+#endif
+                break;
+              }
+#if !defined(LCMSHDRI)
+              case cmsSigLuvData:
+              {
+                source_colorspace=YUVColorspace;
+                source_type=(cmsUInt32Number) TYPE_YUV_16;
+                break;
+              }
+#endif
               case cmsSigRgbData:
               {
+#if defined(LCMSHDRI)
                 source_colorspace=sRGBColorspace;
                 source_type=(cmsUInt32Number) TYPE_RGB_DBL;
+#else
+                source_type=(cmsUInt32Number) TYPE_RGB_16;
+#endif
                 break;
               }
               case cmsSigXYZData:
               {
                 source_colorspace=XYZColorspace;
+#if defined(LCMSHDRI)
                 source_type=(cmsUInt32Number) TYPE_XYZ_DBL;
+#else
+                source_type=(cmsUInt32Number) TYPE_XYZ_16;
+#endif
                 break;
               }
-              default:
+#if !defined(LCMSHDRI)
+              case cmsSigYCbCrData:
               {
-                source_colorspace=UndefinedColorspace;
-                source_type=(cmsUInt32Number) TYPE_RGB_DBL;
+                source_colorspace=YUVColorspace;
+                source_type=(cmsUInt32Number) TYPE_YCbCr_16;
                 break;
               }
+#endif
+              default:
+                ThrowProfileException(ImageError,
+                  "ColorspaceColorProfileMismatch",name);
             }
+            (void) source_colorspace;
             signature=cmsGetPCS(source_profile);
             if (target_profile != (cmsHPROFILE) NULL)
               signature=cmsGetColorSpace(target_profile);
+#if defined(LCMSHDRI)
             target_scale=1.0;
+#endif
             target_channels=3;
             switch (signature)
             {
               case cmsSigCmykData:
               {
                 target_colorspace=CMYKColorspace;
-                target_type=(cmsUInt32Number) TYPE_CMYK_DBL;
                 target_channels=4;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_CMYK_DBL;
                 target_scale=0.01;
-                break;
-              }
-              case cmsSigLabData:
-              {
-                target_colorspace=LabColorspace;
-                target_type=(cmsUInt32Number) TYPE_Lab_DBL;
-                target_scale=0.01;
+#else
+                target_type=(cmsUInt32Number) TYPE_CMYK_16;
+#endif
                 break;
               }
               case cmsSigGrayData:
               {
                 target_colorspace=GRAYColorspace;
-                target_type=(cmsUInt32Number) TYPE_GRAY_DBL;
                 target_channels=1;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_GRAY_DBL;
+#else
+                target_type=(cmsUInt32Number) TYPE_GRAY_16;
+#endif
                 break;
               }
+              case cmsSigLabData:
+              {
+                target_colorspace=LabColorspace;
+#if defined(LCMSHDRI)
+                target_type=(cmsUInt32Number) TYPE_Lab_DBL;
+                target_scale=0.01;
+#else
+                target_type=(cmsUInt32Number) TYPE_Lab_16;
+#endif
+                break;
+              }
+#if !defined(LCMSHDRI)
+              case cmsSigLuvData:
+              {
+                target_colorspace=YUVColorspace;
+                target_type=(cmsUInt32Number) TYPE_YUV_16;
+                break;
+              }
+#endif
               case cmsSigRgbData:
               {
                 target_colorspace=sRGBColorspace;
+#if defined(LCMSHDRI)
                 target_type=(cmsUInt32Number) TYPE_RGB_DBL;
+#else
+                target_type=(cmsUInt32Number) TYPE_RGB_16;
+#endif
                 break;
               }
               case cmsSigXYZData:
               {
                 target_colorspace=XYZColorspace;
+#if defined(LCMSHDRI)
                 target_type=(cmsUInt32Number) TYPE_XYZ_DBL;
+#else
+                target_type=(cmsUInt32Number) TYPE_XYZ_16;
+#endif
                 break;
               }
               default:
-              {
-                target_colorspace=UndefinedColorspace;
-                target_type=(cmsUInt32Number) TYPE_RGB_DBL;
-                break;
-              }
+                ThrowProfileException(ImageError,
+                  "ColorspaceColorProfileMismatch",name);
             }
-            if ((source_colorspace == UndefinedColorspace) ||
-                (target_colorspace == UndefinedColorspace))
-              ThrowProfileException(ImageError,"ColorspaceColorProfileMismatch",
-                name);
             switch (image->rendering_intent)
             {
               case AbsoluteIntent: intent=INTENT_ABSOLUTE_COLORIMETRIC; break;
@@ -1007,9 +1108,11 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
             */
             source_pixels=AcquirePixelThreadSet(image->columns,source_channels);
             target_pixels=AcquirePixelThreadSet(image->columns,target_channels);
-            if ((source_pixels == (double **) NULL) ||
-                (target_pixels == (double **) NULL))
+            if ((source_pixels == (LCMSType **) NULL) ||
+                (target_pixels == (LCMSType **) NULL))
               {
+                target_pixels=DestroyPixelThreadSet(target_pixels);
+                source_pixels=DestroyPixelThreadSet(source_pixels);
                 transform=DestroyTransformThreadSet(transform);
                 ThrowProfileException(ResourceLimitError,
                   "MemoryAllocationFailed",image->filename);
@@ -1044,7 +1147,7 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               register IndexPacket
                 *magick_restrict indexes;
 
-              register double
+              register LCMSType
                 *p;
 
               register PixelPacket
@@ -1066,14 +1169,19 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               p=source_pixels[id];
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                *p++=source_scale*QuantumScale*GetPixelRed(q);
+                *p++=LCMSScaleSource(GetPixelRed(q));
                 if (source_channels > 1)
                   {
-                    *p++=source_scale*QuantumScale*GetPixelGreen(q);
-                    *p++=source_scale*QuantumScale*GetPixelBlue(q);
+                    *p++=LCMSScaleSource(GetPixelGreen(q));
+                    *p++=LCMSScaleSource(GetPixelBlue(q));
                   }
                 if (source_channels > 3)
-                  *p++=source_scale*QuantumScale*GetPixelIndex(indexes+x);
+                  {
+                    *p=LCMSScaleSource(0);
+                    if (indexes != (IndexPacket *) NULL)
+                      *p=LCMSScaleSource(GetPixelIndex(indexes+x));
+                    p++;
+                  }
                 q++;
               }
               cmsDoTransform(transform[id],source_pixels[id],target_pixels[id],
@@ -1082,23 +1190,21 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
               q-=image->columns;
               for (x=0; x < (ssize_t) image->columns; x++)
               {
-                SetPixelRed(q,ClampToQuantum(target_scale*QuantumRange*(*p)));
+                SetPixelRed(q,LCMSScaleTarget(*p));
                 SetPixelGreen(q,GetPixelRed(q));
                 SetPixelBlue(q,GetPixelRed(q));
                 p++;
                 if (target_channels > 1)
                   {
-                    SetPixelGreen(q,ClampToQuantum(target_scale*
-                      QuantumRange*(*p)));
+                    SetPixelGreen(q,LCMSScaleTarget(*p));
                     p++;
-                    SetPixelBlue(q,ClampToQuantum(target_scale*
-                      QuantumRange*(*p)));
+                    SetPixelBlue(q,LCMSScaleTarget(*p));
                     p++;
                   }
                 if (target_channels > 3)
                   {
-                    SetPixelIndex(indexes+x,ClampToQuantum(target_scale*
-                      QuantumRange*(*p)));
+                    if (indexes != (IndexPacket *) NULL)
+                      SetPixelIndex(indexes+x,LCMSScaleTarget(*p));
                     p++;
                   }
                 q++;
@@ -1112,9 +1218,10 @@ MagickExport MagickBooleanType ProfileImage(Image *image,const char *name,
                     proceed;
 
 #if defined(MAGICKCORE_OPENMP_SUPPORT)
-                  #pragma omp critical (MagickCore_ProfileImage)
+                  #pragma omp atomic
 #endif
-                  proceed=SetImageProgress(image,ProfileImageTag,progress++,
+                  progress++;
+                  proceed=SetImageProgress(image,ProfileImageTag,progress,
                     image->rows);
                   if (proceed == MagickFalse)
                     status=MagickFalse;
@@ -1584,6 +1691,36 @@ static void GetProfilesFromResourceBlock(Image *image,
   }
 }
 
+#if defined(MAGICKCORE_XML_DELEGATE)
+static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
+{
+  xmlDocPtr
+    document;
+
+  /*
+    Parse XML profile.
+  */
+  document=xmlReadMemory((const char *) GetStringInfoDatum(profile),(int)
+    GetStringInfoLength(profile),"xmp.xml",NULL,XML_PARSE_NOERROR |
+    XML_PARSE_NOWARNING);
+  if (document == (xmlDocPtr) NULL)
+    return(MagickFalse);
+  xmlFreeDoc(document);
+  return(MagickTrue);
+}
+#else
+static MagickBooleanType ValidateXMPProfile(const StringInfo *profile)
+{
+  char
+    *p;
+
+  p=strcasestr((const char *) GetStringInfoDatum(profile),"x:xmpmeta");
+  if (p != (char *) NULL)
+    p=strcasestr((const char *) GetStringInfoDatum(profile),"rdf:RDF");
+  return(p == (char *) NULL ? MagickFalse : MagickTrue);
+}
+#endif
+
 static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
   const StringInfo *profile,const MagickBooleanType recursive)
 {
@@ -1598,6 +1735,13 @@ static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
   assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if ((LocaleCompare(name,"xmp") == 0) &&
+      (ValidateXMPProfile(profile) == MagickFalse))
+    {
+      (void) ThrowMagickException(&image->exception,GetMagickModule(),
+        ImageWarning,"CorruptImageProfile","`%s'",name);
+      return(MagickTrue);
+    }
   if (image->profiles == (SplayTreeInfo *) NULL)
     image->profiles=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
       DestroyProfile);
@@ -2043,13 +2187,15 @@ static MagickBooleanType SyncExifProfile(Image *image, StringInfo *profile)
         case 0x011a:
         {
           (void) WriteProfileLong(endian,(size_t) (image->x_resolution+0.5),p);
-          (void) WriteProfileLong(endian,1UL,p+4);
+          if (number_bytes == 8)
+            (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x011b:
         {
           (void) WriteProfileLong(endian,(size_t) (image->y_resolution+0.5),p);
-          (void) WriteProfileLong(endian,1UL,p+4);
+          if (number_bytes == 8)
+            (void) WriteProfileLong(endian,1UL,p+4);
           break;
         }
         case 0x0112:
