@@ -104,6 +104,7 @@ static MagickBooleanType MontageUsage(void)
       "  -flip                flip image in the vertical direction\n"
       "  -flop                flop image in the horizontal direction\n"
       "  -frame geometry      surround image with an ornamental border\n"
+      "  -layers method       optimize, merge, or compare image layers\n"
       "  -monochrome          transform image to black and white\n"
       "  -polaroid angle      simulate a Polaroid picture\n"
       "  -resize geometry     resize the image\n"
@@ -133,6 +134,7 @@ static MagickBooleanType MontageUsage(void)
       "  -compress type       type of pixel compression when writing the image\n"
       "  -define format:option\n"
       "                       define one or more image format options\n"
+      "  -delay value         display the next image after pausing\n"
       "  -density geometry    horizontal and vertical density of the image\n"
       "  -depth value         image depth\n"
       "  -display server      query font from this X server\n"
@@ -766,6 +768,17 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageInvalidArgumentException(option,argv[i]);
             break;
           }
+        if (LocaleCompare("delay",option+1) == 0)
+          {
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            if (IsGeometry(argv[i]) == MagickFalse)
+              ThrowMontageInvalidArgumentException(option,argv[i]);
+            break;
+          }
         if (LocaleCompare("density",option+1) == 0)
           {
             if (*option == '+')
@@ -1119,6 +1132,22 @@ WandExport MagickBooleanType MontageImageCommand(ImageInfo *image_info,
               ThrowMontageException(OptionError,"MissingArgument",option);
             break;
           }
+        if (LocaleCompare("layers",option+1) == 0)
+          {
+            ssize_t
+              type;
+
+            if (*option == '+')
+              break;
+            i++;
+            if (i == (ssize_t) argc)
+              ThrowMontageException(OptionError,"MissingArgument",option);
+            type=ParseCommandOption(MagickLayerOptions,MagickFalse,argv[i]);
+            if (type < 0)
+              ThrowMontageException(OptionError,"UnrecognizedLayerMethod",
+                argv[i]);
+            break;
+          }
         if (LocaleCompare("limit",option+1) == 0)
           {
             char
